@@ -9,10 +9,10 @@ import (
 
 	"github.com/pborman/getopt/v2"
 
+	"sigsum.org/sigsum-go/internal/ui"
 	"sigsum.org/sigsum-go/internal/version"
 	"sigsum.org/sigsum-go/pkg/crypto"
 	"sigsum.org/sigsum-go/pkg/key"
-	"sigsum.org/sigsum-go/pkg/policy"
 	"sigsum.org/sigsum-go/pkg/proof"
 )
 
@@ -21,13 +21,22 @@ type Settings struct {
 	proofFile  string
 	submitKey  string
 	policyFile string
+	policyName string
 }
 
 func main() {
 	log.SetFlags(0)
 	var settings Settings
 	settings.parse(os.Args)
-	submitKeys, err := key.ReadPublicKeysFile(settings.submitKey)
+	var submitKeys map[crypto.Hash]crypto.PublicKey
+	policyNameFromPubKeys := ""
+	var err error
+	// Care about policy from pubkeys only if no policy option was specified
+	if settings.policyFile == "" && settings.policyName == "" {
+		submitKeys, policyNameFromPubKeys, err = key.ReadPublicKeysFileWithPolicy(settings.submitKey)
+	} else {
+		submitKeys, err = key.ReadPublicKeysFile(settings.submitKey)
+	}
 	if err != nil {
 		log.Fatal(err)
 	}
@@ -38,18 +47,25 @@ func main() {
 
 	f, err := os.Open(settings.proofFile)
 	if err != nil {
-		log.Fatalf("opening file %q failed: %v", settings.proofFile, err)
+		log.Fatalf("Opening file %q failed: %v", settings.proofFile, err)
 	}
 	var pr proof.SigsumProof
 	if err := pr.FromASCII(f); err != nil {
-		log.Fatalf("invalid proof: %v", err)
+		log.Fatalf("Invalid proof: %v", err)
 	}
-	policy, err := policy.ReadPolicyFile(settings.policyFile)
+	policy, err := ui.SelectPolicy(ui.PolicyParams{
+		File:           settings.policyFile,
+		Name:           settings.policyName,
+		NameFromPubKey: policyNameFromPubKeys,
+	})
 	if err != nil {
-		log.Fatalf("failed to create policy: %v", err)
+		log.Fatalf("Failed to select policy: %v", err)
+	}
+	if policy == nil {
+		log.Fatalf("A policy must be specified, either in pubkey file or using -p or -P")
 	}
 	if err := pr.Verify(&msg, submitKeys, policy); err != nil {
-		log.Fatalf("sigsum proof failed to verify: %v", err)
+		log.Fatalf("Sigsum proof failed to verify: %v", err)
 	}
 }
 
@@ -65,7 +81,8 @@ policy.  The message to be verified is read on stdin.
 	versionFlag := false
 	set.FlagLong(&s.rawHash, "raw-hash", 0, "Input has already been hashed and formatted as 32 octets or a hex string")
 	set.FlagLong(&s.submitKey, "key", 'k', "Submitter public keys, one per line in OpenSSH format", "key-file").Mandatory()
-	set.FlagLong(&s.policyFile, "policy", 'p', "Trust policy defining logs, witnesses, and a quorum rule", "policy-file").Mandatory()
+	set.FlagLong(&s.policyFile, "policy", 'p', "Trust policy file defining logs, witnesses, and a quorum rule", "policy-file")
+	set.FlagLong(&s.policyName, "named-policy", 'P', "Use a named trust policy defining logs, witnesses, and a quorum rule", "policy-name")
 	set.FlagLong(&help, "help", 0, "Show usage message and exit")
 	set.FlagLong(&versionFlag, "version", 'v', "Show software version and exit")
 	err := set.Getopt(args, nil)
@@ -85,8 +102,11 @@ policy.  The message to be verified is read on stdin.
 		fmt.Fprint(os.Stderr, usage)
 		os.Exit(1)
 	}
+	if len(s.policyName) > 0 && len(s.policyFile) > 0 {
+		log.Fatal("The -P (--named-policy) and -p (--policy) options are mutually exclusive.")
+	}
 	if set.NArgs() != 1 {
-		log.Fatalf("no proof given on command line")
+		log.Fatalf("No proof given on command line")
 	}
 	s.proofFile = set.Arg(0)
 }
